@@ -8,6 +8,7 @@ import os
 import platform
 import subprocess
 import sys
+import traceback
 
 try:
     from configparser import Error as ConfigParserError, SafeConfigParser
@@ -21,7 +22,16 @@ from distutils.util import strtobool
 __author__ = 'Igor Davydenko'
 __license__ = 'BSD License'
 __script__ = 'bootstrapper'
-__version__ = '0.2.1'
+__version__ = '0.2.2'
+
+
+IS_PY3 = sys.version_info[0] == 3
+IS_WINDOWS = platform.system() == 'Windows'
+
+iteritems = lambda seq: seq.items() if IS_PY3 else seq.iteritems()
+iterkeys = lambda seq: seq.keys() if IS_PY3 else seq.iterkeys()
+safe_path = lambda value: value.replace('/', os.sep) if IS_WINDOWS else value
+string_types = (bytes, str) if IS_PY3 else basestring
 
 
 CONFIG = {
@@ -31,17 +41,13 @@ CONFIG = {
         'quiet': False,
     },
     'pip': {
-        'download_cache': '~/.{0}/pip-cache/'.format(__script__),
+        'download_cache': safe_path(os.path.expanduser(
+            os.path.join('~', '.{0}'.format(__script__), 'pip-cache')
+        )),
     },
     'virtualenv': {},
 }
 DEFAULT_CONFIG = 'bootstrap.cfg'
-IS_PY3 = sys.version_info[0] == 3
-IS_WINDOWS = platform.system() == 'Windows'
-
-iteritems = lambda seq: seq.items() if IS_PY3 else seq.iteritems()
-iterkeys = lambda seq: seq.keys() if IS_PY3 else seq.iterkeys()
-string_types = (bytes, str) if IS_PY3 else basestring
 
 
 def check_pre_requirements(pre_requirements):
@@ -107,7 +113,7 @@ def error(message, code=None):
     """
     Print error message and exit with error code.
     """
-    print('{0}. Exit...'.format(message.rstrip('.')))
+    print('ERROR: {0}. Exit...'.format(message.rstrip('.')))
     sys.exit(code or 1)
 
 
@@ -143,35 +149,56 @@ def main(*args):
     # Create parser, read arguments from direct input or command line
     args = parse_args(args or sys.argv[1:])
 
-    # Initialize bootstrapper instance Read current config from file
-    config = read_config(args.config, args)
-    bootstrap = config[__script__]
+    try:
+        # Initialize bootstrapper instance Read current config from file
+        config = read_config(args.config, args)
+        bootstrap = config[__script__]
 
-    # Check pre-requirements
-    check_pre_requirements(bootstrap['pre_requirements'])
+        # Check pre-requirements
+        check_pre_requirements(bootstrap['pre_requirements'])
 
-    # Create virtual environment
-    env_args = prepare_args(config['virtualenv'], bootstrap)
-    create_env(bootstrap['env'],
-               env_args,
-               bootstrap['recreate'],
-               bootstrap['quiet'])
+        # Create virtual environment
+        env_args = prepare_args(config['virtualenv'], bootstrap)
+        create_env(bootstrap['env'],
+                   env_args,
+                   bootstrap['recreate'],
+                   bootstrap['quiet'])
 
-    # And install library or project here
-    pip_args = prepare_args(config['pip'], bootstrap)
-    install(bootstrap['env'],
-            bootstrap['requirements'],
-            pip_args,
-            bootstrap['quiet'])
+        # And install library or project here
+        pip_args = prepare_args(config['pip'], bootstrap)
+        install(bootstrap['env'],
+                bootstrap['requirements'],
+                pip_args,
+                bootstrap['quiet'])
 
-    # Run post-bootstrap hook
-    run_hook(bootstrap['hook'], bootstrap, bootstrap['quiet'])
+        # Run post-bootstrap hook
+        run_hook(bootstrap['hook'], bootstrap, bootstrap['quiet'])
 
-    # All OK!
-    if not bootstrap['quiet']:
-        print('All OK!')
+        # All OK!
+        if not bootstrap['quiet']:
+            print('All OK!')
+    except BaseException as err:
+        filename = safe_path(os.path.expanduser(
+            os.path.join('~',
+                         '.{0}'.format(__script__),
+                         '{0}.log'.format(__script__))
+        ))
 
-    return True
+        with open(filename, 'a+') as handler:
+            traceback.print_exc(file=handler)
+
+        message = ('User aborted workflow'
+                   if isinstance(err, KeyboardInterrupt)
+                   else 'Unexpected error catched')
+        print('ERROR: {0}. Exit...'.format(message), file=sys.stderr)
+        print('Full log stored to ~/.{0}/{0}.log'.format(__script__),
+              file=sys.stderr)
+
+        # True means error happened, exit code: 1
+        return True
+
+    # False means everything went alright, exit code: 0
+    return False
 
 
 def parse_args(args):
@@ -222,15 +249,12 @@ def pip_cmd(venv, cmd, **kwargs):
     """
     Run pip command in given virtual environment.
     """
-    prefix = ''
-
-    if IS_WINDOWS:
-        prefix = 'python '
-        pip_path = os.path.join(venv, 'Scripts', 'pip-script.py')
-    else:
-        pip_path = os.path.join(venv, 'bin', 'pip')
-
-    return run_cmd('{0}{1} {2}'.format(prefix, pip_path, cmd), **kwargs)
+    pip_path = os.path.join(safe_path(venv),
+                            'Scripts' if IS_WINDOWS else 'bin',
+                            'pip')
+    return (pip_path
+            if kwargs.pop('return_path', False)
+            else run_cmd('{0} {1}'.format(pip_path, cmd), **kwargs))
 
 
 def prepare_args(config, bootstrap):
@@ -241,6 +265,7 @@ def prepare_args(config, bootstrap):
     environ = dict(copy.deepcopy(os.environ))
 
     data = {'env': bootstrap['env'],
+            'pip': pip_cmd(bootstrap['env'], '', return_path=True),
             'requirements': bootstrap['requirements']}
     environ.update(data)
 
@@ -266,7 +291,8 @@ def read_config(filename, args):
     config = defaultdict(dict)
     converters = {
         __script__: {
-            'pre_requirements': operator.methodcaller('split', ' ')
+            'env': safe_path,
+            'pre_requirements': operator.methodcaller('split', ' '),
         },
     }
     default = copy.deepcopy(CONFIG)
@@ -394,4 +420,4 @@ def which(executable):
 
 
 if __name__ == '__main__':
-    sys.exit(int(not main()))
+    sys.exit(int(main()))
